@@ -128,6 +128,10 @@ void parseparams() {
 
 		/* get screen and type for the pier */
 		OPTIONAL_PARAM(&param->subparams, "screen", int, screen, 0);
+		if (screen < 0 || screen >= ScreenCount(display)) {
+			PWARN("invalid screen number %d", screen);
+			continue;
+		}
 		if (strcmp(param->value, "horizontal") == 0)
 			type = PT_HORIZ;
 		else if (strcmp(param->value, "vertical") == 0)
