@@ -53,14 +53,17 @@ int pier_init(int size, pixmap_t *pixmap) {
 
 /* shutdown the pier system */
 void pier_shutdown() {
-	comtab_t *comtab;
+	comtab_t *comtab, *tmptab;
 	pier_t *pier;
 
 	/* free any comtabs that may still be here */
-	while (!LIST_EMPTY(&comtab_list)) {
-		comtab = LIST_FIRST(&comtab_list);
+	comtab = LIST_FIRST(&comtab_list);
+	while (comtab) {
+		tmptab = LIST_NEXT(comtab, ct_list);
 		free(comtab);
+		comtab = tmptab;
 	}
+	LIST_INIT(&comtab_list);
 
 	/* delete every pier */
 	while (!SLIST_EMPTY(&pier_list)) {
