@@ -34,7 +34,12 @@ static struct menuscr {
 } *menuscr;
 
 /* font to use */
+#ifdef I18N
+static XFontSet	menufont;
+static XFontSetExtents *menufont_extents;
+#else
 static XFontStruct	*menufont	= NULL;
+#endif
 
 /* decoration group */
 static dgroup_t		*menu_dgroup	= NULL;
@@ -63,24 +68,40 @@ int menu_init(char *fontname, dgroup_t *dgroup, pixmap_t *bullet) {
 	 * we give up.
 	 */
 	if (fontname) {
+#ifdef I18N
+		menufont = XLoadQueryFontSet(display, fontname);
+#else
 		menufont = XLoadQueryFont(display, fontname);
+#endif
 		if (menufont)
 			goto gotfont;
 		PWARN("unable to get requested menu_font, trying default");
 	}
 	PWARN("using default font");
+#ifdef I18N
+	menufont = XLoadQueryFontSet(display,
+		"-*-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*");
+#else
 	menufont = XLoadQueryFont(display,
 		"-*-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*");
+#endif
 	if (menufont)
 		goto gotfont;
 	PWARN("failed to load default font; trying 'fixed' as last resort");
+#ifdef I18N
+	menufont = XLoadQueryFontSet(display, "fixed");
+#else
 	menufont = XLoadQueryFont(display, "fixed");
+#endif
 	if (menufont)
 		goto gotfont;
 	PWARN("failed to load font 'fixed', giving up on menus");
 	return -1;
 gotfont:
 
+#ifdef I18N
+        menufont_extents = XExtentsOfFontSet(menufont);
+#endif
 	/* get the per-screen structure */
 	cnt = ScreenCount(display);
 	menuscr = calloc(cnt, sizeof(struct menuscr));
@@ -94,9 +115,14 @@ gotfont:
 		 */
 		gcvalues.foreground = WhitePixel(display, i);
 		gcvalues.background = BlackPixel(display, i);
+#ifdef I18N	    
+		menuscr[i].menugc = XCreateGC(display, RootWindow(display, i),
+			GCForeground | GCBackground, &gcvalues);
+#else	    
 		gcvalues.font = menufont->fid;
 		menuscr[i].menugc = XCreateGC(display, RootWindow(display, i),
 			GCFont | GCForeground | GCBackground, &gcvalues);
+#endif
 	}
 
 	menu_dgroup = dgroup;
@@ -116,7 +142,11 @@ void menu_shutdown() {
 		free(menuscr);
 	}
 	if (menufont)
+#ifdef I18N
+		XFreeFontSet(display, menufont);
+#else
 		XFreeFont(display, menufont);
+#endif
 }
 
 /* create a menu */
@@ -210,8 +240,13 @@ void menu_size(menu_t *menu) {
 	width = MENU_MINWIDTH;
 	height = MENU_YBORDER * 2;
 	for (i = 0; i < menu->entry_count; i++) {
+#ifdef I18N
+		height += menufont_extents->max_ink_extent.height;
+		strwidth = XmbTextEscapement(menufont, menu->entries[i]->text, strlen(menu->entries[i]->text));
+#else	    
 		height += menufont->ascent + menufont->descent;
 		strwidth = XTextWidth(menufont, menu->entries[i]->text, strlen(menu->entries[i]->text));
+#endif	    
 		if (submenu_bullet && menu->entries[i]->type == ET_SUBMENU)
 			strwidth += submenu_bullet->width;
 		if (width < strwidth)
@@ -327,7 +362,11 @@ static void menu_drawent(menu_t *menu, client_t *client, int i, int y) {
 
 	if (submenu_bullet && menu->entries[i]->type == ET_SUBMENU) {
 		tmpx = client->width - submenu_bullet->width;
+#ifdef I18N
+		tmpy = y + (((menufont_extents->max_ink_extent.height) / 2) - submenu_bullet->height / 2);
+#else
 		tmpy = y + (((menufont->ascent + menufont->descent) / 2) - submenu_bullet->height / 2);
+#endif	    
 	
 		XSetClipMask(display, menuscr[client->screen->num].menugc,
 			submenu_bullet->shapemasks[client->screen->num]);
@@ -337,8 +376,13 @@ static void menu_drawent(menu_t *menu, client_t *client, int i, int y) {
 			submenu_bullet->height, tmpx, tmpy);
 		XSetClipMask(display, menuscr[client->screen->num].menugc, None);
 	}
+#ifdef I18N
+	XmbDrawString(display, client->window, menufont, menuscr[client->screen->num].menugc,
+		MENU_XBORDER, y + menufont_extents->max_logical_extent.height * 4 / 5, menu->entries[i]->text, strlen(menu->entries[i]->text));
+#else    
 	XDrawString(display, client->window, menuscr[client->screen->num].menugc,
 		MENU_XBORDER, y + menufont->ascent, menu->entries[i]->text, strlen(menu->entries[i]->text));
+#endif
 }
 
 /* draw a menu; entidx is for use during menu_interact */
@@ -353,7 +397,11 @@ void menu_expose(menu_t *menu, client_t *client, XExposeEvent *e) {
 	y = MENU_YBORDER;
 	last = first = -1;
 	for (i = 0; i < menu->entry_count; i++) {
+#ifdef I18N    
+		y += menufont_extents->max_ink_extent.height;
+#else	    
 		y += menufont->ascent + menufont->descent;
+#endif
 		if (y > e->y && first == -1)
 			first = i - 1;
 		if (y > e->y + e->height && last == -1)
@@ -363,10 +411,18 @@ void menu_expose(menu_t *menu, client_t *client, XExposeEvent *e) {
 	/* now we draw the lines that need it */
 	if (first < 0) first = 0;
 	if (last == -1) last = menu->entry_count - 1;
+#ifdef I18N
+    	y = MENU_YBORDER + (menufont_extents->max_ink_extent.height) * first;
+#else    
 	y = MENU_YBORDER + (menufont->ascent + menufont->descent) * first;
+#endif    
 	for (i = first; i < last + 1; i++) {
 		menu_drawent(menu, client, i, y);
+#ifdef I18N	    
+		y += menufont_extents->max_ink_extent.height;
+#else	    
 		y += menufont->ascent + menufont->descent;
+#endif	    
 	}
 }
 
@@ -409,8 +465,10 @@ static void menu_close(menu_t *menu, client_t *client) {
 	int snum = client->screen->num;
 
 	/* close any open submenus */
-	if (menu->active_sub[snum])
+	if (menu->active_sub[snum]) {
 		menu_close(menu->active_sub[snum], menu->active_sub[snum]->client[snum]);
+		menu->active_sub[snum] = NULL;
+	}
 	if (client->workspace) {
 		desktop_rm_client(client);
 		workspace_rm_client(client);
@@ -421,9 +479,15 @@ static void menu_close(menu_t *menu, client_t *client) {
 
 /* draw (or erase) a highlight */
 static void drawhighlight(client_t *client, int idx) {
+#ifdef I18N    
+	XFillRectangle(display, client->window, client->screen->xorgc, 2,
+		idx * (menufont_extents->max_ink_extent.height) + MENU_YBORDER,
+		client->width - 5, menufont_extents->max_ink_extent.height);
+#else    
 	XFillRectangle(display, client->window, client->screen->xorgc, 2,
 		idx * (menufont->ascent + menufont->descent) + MENU_YBORDER,
 		client->width - 5, menufont->ascent + menufont->descent);
+#endif
 }
 
 /* open submenus or close them as neccessary while dragging across */
@@ -433,9 +497,15 @@ static void passopen(menu_t *menu, client_t *client, int entidx, int *oldentidx,
 			if (menu->active_sub[snum])
 				menu_close(menu, menu->active_sub[snum]->client[snum]);
 			menu->active_sub[snum] = menu->submenus[menu->entries[entidx]->dat.submenu];
+#ifdef I18N		    
+			menu_open(menu->active_sub[snum]->client[snum],
+				client->x + FULLWIDTH(client),
+				client->y + MENU_YBORDER + (entidx * (menufont_extents->max_ink_extent.height)));
+#else				  
 			menu_open(menu->active_sub[snum]->client[snum],
 				client->x + FULLWIDTH(client),
 				client->y + MENU_YBORDER + (entidx * (menufont->ascent + menufont->descent)));
+#endif				  
 		}
 		*oldentidx = -1;
 	} else {
@@ -468,8 +538,13 @@ static void menu_interact(menu_t *menu, client_t *client, XButtonEvent *butev) {
 
 	/* set the entry index var if we got a event structure */
 	if (butev) {
+#ifdef I18N	    
+		oldentidx = entidx = (butev->y - MENU_YBORDER) 
+			/ (menufont_extents->max_ink_extent.height);
+#else	    
 		oldentidx = entidx = (butev->y - MENU_YBORDER) 
 			/ (menufont->ascent + menufont->descent);
+#endif	    
 		if (entidx >= 0 && entidx < menu->entry_count) {
 			XSync(display, 0);
 			while (XCheckMaskEvent(display, ExposureMask, &event))
@@ -521,7 +596,11 @@ static void menu_interact(menu_t *menu, client_t *client, XButtonEvent *butev) {
 				break;
 			}
 
+#ifdef I18N
+			entidx = (ypos - MENU_YBORDER) / (menufont_extents->max_ink_extent.height);
+#else		    
 			entidx = (ypos - MENU_YBORDER) / (menufont->ascent + menufont->descent);
+#endif		    
 			if (entidx != oldentidx) {
 				if (oldentidx != -1)
 					drawhighlight(client, oldentidx);
