@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2001 Jordan DeLong
+ * Copyright (c) 2001, 2002 Jordan DeLong
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -112,6 +112,8 @@ static __inline void client_decorate(client_t *client, dgroup_t *dgroup) {
 		FULLWIDTH(client), FULLHEIGHT(client), 0, CopyFromParent, CopyFromParent,
 		CopyFromParent, CWOverrideRedirect | CWBackPixel, &attr);
 	XSetWindowBorderWidth(display, client->window, 0);
+
+	/* actual decor window creation (decor_decorate) posponed 'till later in add */
 	decor_decorate(client);
 	client_shape(client);
 }
@@ -128,15 +130,13 @@ static __inline void client_dograbs(client_t *client) {
 			XGrabButton(display, AnyButton, AnyModifier, client->frame,
 				1, ButtonPressMask, GrabModeSync, GrabModeAsync, None, None);
 		else
-			XGrabButton(display, AnyButton, options.mouse_modifier, client->frame,
-				1, ButtonPressMask, GrabModeSync, GrabModeAsync, None, None);
+			buttongrabhack(client->frame, AnyButton, options.mouse_modifier);
 		break;
 	case FOCUS_POINTER:
 		mask |= LeaveWindowMask;
 	case FOCUS_SLOPPY:
 		mask |= EnterWindowMask;
-		XGrabButton(display, AnyButton, options.mouse_modifier, client->frame,
-			1, ButtonPressMask, GrabModeSync, GrabModeAsync, None, None);
+		buttongrabhack(client->frame, AnyButton, options.mouse_modifier);
 		break;
 	}
 	XSelectInput(display, client->frame, mask);
@@ -164,6 +164,11 @@ client_t *client_add(screen_t *screen, Window w,
 	Window dumwin;
 	client_t *client = NULL;
 	long supplied;
+#ifdef I18N
+	XTextProperty text_prop;
+	char **cl;
+	int n;
+#endif
 
 	/*
 	 * we need to do a grab so that it's not possible for the window
@@ -202,7 +207,25 @@ client_t *client_add(screen_t *screen, Window w,
 	client->wmhints = XGetWMHints(display, client->window);
 	XGetWMNormalHints(display, client->window, &client->normalhints, &supplied);
 	XGetClassHint(display, client->window, &client->classhint);
+#ifdef I18N
+	if (XGetWMName(display, client->window, &text_prop) != 0) {        
+		if (text_prop.encoding == XA_STRING) {
+			client->store_name = (char *)text_prop.value;
+		} else {
+			XmbTextPropertyToTextList(display, &text_prop, &cl, &n);
+			if (cl) {
+				client->store_name = strdup(cl[0]);
+				XFreeStringList(cl);
+			} else {
+				client->store_name = strdup("NoName");
+			}
+		}
+	} else {
+		client->store_name = strdup("NoName");
+	}
+#else
 	XFetchName(display, client->window, &client->store_name);
+#endif
 	if (XGetTransientForHint(display, client->window, &dumwin))
 		client->flags.transient = 1;
 
@@ -224,6 +247,9 @@ client_t *client_add(screen_t *screen, Window w,
 		client->dgroup->left_space, client->dgroup->top_space);
 	client_gravitate(client);
 
+	/* lower the client in the frame window so decoration can cover it */
+	XLowerWindow(display, client->window);
+
 	/* link it up to the all-clients list */
 	LIST_INSERT_HEAD(&client_list, client, c_list);
 
