@@ -29,7 +29,11 @@
 #include "wm.h"
 
 /* font for titles */
-XFontStruct	*titlefont;
+#ifdef I18N
+ XFontSet   titlefont;
+#else
+ XFontStruct	*titlefont;
+#endif
 
 /* list of decoration units */
 static SLIST_HEAD(, decor) decor_list = SLIST_HEAD_INITIALIZER(&decor_list);
@@ -37,11 +41,18 @@ static SLIST_HEAD(, decor) decor_list = SLIST_HEAD_INITIALIZER(&decor_list);
 /* get our titlebar font */
 void decor_init() {
 	char *deffont = "fixed";
-
+#ifdef I18N
+	titlefont = XLoadQueryFontSet(display, options.title_font ? options.title_font : deffont);
+#else
 	titlefont = XLoadQueryFont(display, options.title_font ? options.title_font : deffont);
+#endif
 	if (!titlefont) {
 		if (options.title_font) {
+#ifdef I18N
+			titlefont = XLoadQueryFontSet(display, deffont);
+#else
 			titlefont = XLoadQueryFont(display, deffont);
+#endif
 			if (!titlefont)
 				errx(1, "unable to load fallback titlefont 'fixed'");
 		} else
@@ -54,7 +65,11 @@ void decor_shutdown() {
 	decor_t *decor;
 
 	/* free the title font */
+#ifdef I18N
+	XFreeFontSet(display, titlefont);
+#else
 	XFreeFont(display, titlefont);
+#endif
 
 	/* free the list of decor_t */
 	while (!SLIST_EMPTY(&decor_list)) {
@@ -147,11 +162,15 @@ static void decor_getdims(client_t *client, decor_t *decor, int *x, int *y,
 		*pheight += decor->lenmod;
 		*y += client->height * decor->offsetmult;
 		*pheight += client->height * decor->lenmult;
+
+		*x += decor->soffset;
 	} else {
 		*x += decor->offset;
 		*pwidth += decor->lenmod;
 		*x += client->width * decor->offsetmult;
 		*pwidth += client->width * decor->lenmult;
+
+		*y += decor->soffset;
 	}
 
 	/* finally, make sure that the width/height are valid */
@@ -167,7 +186,7 @@ static void decor_getdims(client_t *client, decor_t *decor, int *x, int *y,
 	XShapeCombineRectangles(display, client->frame, ShapeBounding, 0, 0, rect,		\
 		1, ShapeUnion, Unsorted);							\
 } while (0)
-	
+
 /* shape a decoration unit onto this client's frame */
 static void decor_shapeunit(client_t *client, decor_t *decor, int idx) {
 	XRectangle rect[1];
@@ -435,6 +454,8 @@ void decor_handlepress(client_t *client, decor_t *decor, XButtonEvent *e) {
 
 	if (e->button == Button3)
 		action = decor->rclick_action;
+	else if (e->button == Button2)
+		action = decor->mclick_action;
 	else
 		action = decor->lclick_action;
 
@@ -477,6 +498,11 @@ void decor_unfocus(client_t *client) {
 /* handle exposes on decoration */
 void decor_expose(client_t *client, decor_t *decor, XExposeEvent *e) {
 	int idx;
+#ifdef I18N
+	XFontSetExtents *extent;
+
+	extent = XExtentsOfFontSet(titlefont);
+#endif   
 
 	/* we are just drawing the whole thing, so we only use count == 0 */
 	if (e && e->count > 0)
@@ -492,8 +518,13 @@ void decor_expose(client_t *client, decor_t *decor, XExposeEvent *e) {
 	 * actually need to draw is kinda more trouble than it's worth.
 	 */
 	if (client->store_name)
+#ifdef I18N
+		XmbDrawString(display, client->decorwin[idx], titlefont, client->screen->titlegc, client->dgroup->title_x,
+			client->dgroup->title_y + (extent->max_logical_extent.height * 4 / 5), client->store_name, strlen(client->store_name));
+#else
 		XDrawString(display, client->decorwin[idx], client->screen->titlegc, client->dgroup->title_x,
 			client->dgroup->title_y + titlefont->ascent, client->store_name, strlen(client->store_name));
+#endif
 }
 
 /* if a window's title (store_name) changed, we hear about it here */
@@ -508,3 +539,4 @@ void decor_titlechange(client_t *client) {
 		}
 	}
 }
+
