@@ -73,13 +73,16 @@ struct decor {
 	int	type;		/* type of decoration */
 	int	edge;		/* edge to put this decor on (top, left, etc) */
 
-	int	offset;		/* pixel offset for placement */
+	int	offset;		/* pixel offset for placement in primary direction */
+	int	soffset;	/* pixel offset in secondary direction; used to allow
+				 * overlapping the client's area */
 	int	lenmod;		/* modification to the length of decor */
 	double	offsetmult;	/* factor * the total len of the edge to offset by */
 	double	lenmult;	/* factor * the total len of the edge to size by */
 
 	int	lclick_action;	/* what to do when pressed, or if button when clicked */
 	int	rclick_action;	/* same for the right click */
+	int	mclick_action;	/* same for the middle click */
 
 	pixmap_t 	*pixmap;	/* pixmap struct for the decoration unit when focused */
 	pixmap_t	*focpixmap;	/* pixmap for when unit is focused */
@@ -93,7 +96,11 @@ struct decor {
 	SLIST_ENTRY(decor) d_list;
 };
 
+#ifdef I18N
+extern XFontSet titlefont;
+#else
 extern XFontStruct *titlefont;
+#endif
 
 void decor_init();
 void decor_shutdown();
