@@ -58,7 +58,7 @@ static void event_configure_request(XEvent *event) {
 	if (XFindContext(display, e->window, client_context, (XPointer *) &client))
 		client = NULL;
 
-	/* 
+	/*
 	 * we need to respond to ConfigureRequests of windows that
 	 * haven't yet been mapped, and thus don't have client_t structs
 	 * in our list.
@@ -225,7 +225,7 @@ static void event_button_press(XEvent *event) {
 		 * XXX: the buttons that do which for mouse_modifier should
 		 * be configurable.
 		 */
-		if (e->state == options.mouse_modifier) {
+		if ((e->state & ~(numlock_mask | scroll_mask | LockMask)) == options.mouse_modifier) {
 			if (e->button == Button3)
 				action_resize(client);
 			else
@@ -326,7 +326,7 @@ static void event_client_message(XEvent *event) {
 	 * for iconification.
 	 */
 	if (e->message_type == WM_CHANGE_STATE && e->format == 32
-			&& e->data.l[0] == IconicState 
+			&& e->data.l[0] == IconicState
 			&& client->state != IconicState)
 		action_iconify(client);
 }
@@ -336,6 +336,11 @@ static void event_property_notify(XEvent *event) {
 	XPropertyEvent *e = &event->xproperty;
 	client_t *client;
 	long supplied;
+#ifdef I18N
+	XTextProperty text_prop;
+	char **cl;
+	int n;
+#endif
 
 	/* get associated client, the event must be for it's window */
 	if (XFindContext(display, e->window, client_context, (XPointer *) &client))
@@ -362,7 +367,25 @@ static void event_property_notify(XEvent *event) {
 		case XA_WM_NAME:
 			if (client->store_name)
 				XFree(client->store_name);
+#ifdef I18N
+			if (XGetWMName(display, client->window, &text_prop) != 0) {
+				if (text_prop.encoding == XA_STRING) {
+					client->store_name = (char *)text_prop.value;
+				} else {
+					XmbTextPropertyToTextList(display, &text_prop, &cl, &n);
+					if (cl) {
+						client->store_name = strdup(cl[0]);
+						XFreeStringList(cl);
+					} else {
+						client->store_name = "NoName";
+					}
+				}
+			} else {
+				client->store_name = "NoName";
+			}
+#else
 			XFetchName(display, client->window, &client->store_name);
+#endif
 
 			/*
 			 * XXX: need a callback to let plugins know
@@ -414,7 +437,7 @@ static void event_leave_notify(XEvent *event) {
 	 * a leave notify if the user iconifies a window, and the window
 	 * will have left it's workspace, and thus already be unfocused.
 	 */
-	if (client->workspace) 
+	if (client->workspace)
 		focus_unfocus(client);
 }
 
@@ -478,11 +501,11 @@ void event_handle(XEvent *e) {
 
 	/*
 	 * if a handler exists in the table of event handlers,
-	 * dispatch the event to it.	
+	 * dispatch the event to it.
 	 */
 	if (e->type == shape_base + ShapeNotify)
 		event_shape_notify(e);
-	else if (handlers[e->type] != NULL) 
+	else if (handlers[e->type] != NULL)
 		handlers[e->type](e);
 
 }
