@@ -109,7 +109,7 @@ void focus_setfocused(client_t *client) {
 	 * if it is on some other desktop/workspace.
 	 */
 	if (client->workspace == client->screen->desktop->current_space)
-		focus_client(client);	
+		focus_client(client);
 	else
 		client->workspace->focused = client;
 }
@@ -138,8 +138,7 @@ void focus_client(client_t *client) {
 	/* prepare this client to recieve the focus */
 	if (options.focus == FOCUS_CLICK) {
 		XUngrabButton(display, AnyButton, AnyModifier, client->frame);
-		XGrabButton(display, AnyButton, options.mouse_modifier, client->frame,
-			1, ButtonPressMask, GrabModeAsync, GrabModeAsync, None, None);
+		buttongrabhack(client->frame, AnyButton, options.mouse_modifier);
 	}
 
 	/* now we actually focus it */
