@@ -72,7 +72,7 @@ enum {
 /* structure for a keybinding */
 struct keybind {
 	int	keycode;	/* keycode we are binding */
-	int	modifiers;	/* modifiers for the bind */
+	u_int	modifiers;	/* modifiers for the bind */
 	int	action;		/* what the key does */
 
 	union {
@@ -87,8 +87,12 @@ struct keybind {
 	SLIST_ENTRY(keybind) k_list;
 };
 
+/* grabbing hack stuff */
+u_int numlock_mask, scroll_mask;
+void buttongrabhack(Window win, int button, u_int modifiers);
+
 void keys_shutdown();
-keybind_t *keys_add(int keycode, int modifiers, int action, void *dat);
+keybind_t *keys_add(int keycode, u_int modifiers, int action, void *dat);
 void keys_grab(screen_t *screen);
 void keys_press(screen_t *screen, XKeyEvent *e);
 
