@@ -84,6 +84,7 @@ static RETSIGTYPE handle_sigusr1(int no) {
 
 /* setup signals, open display, parse the rcfile, and then go to our loop */
 int main(int argc, char **argv) {
+
 	/* if -v we just print version and exit */
 	if (argc > 1 && strcmp(argv[1], "-v") == 0) {
 		printf("golem-" VERSION "\n"
@@ -92,10 +93,12 @@ int main(int argc, char **argv) {
 		exit(0);
 	}
 
+#ifdef I18N
+	setlocale(LC_CTYPE,"");
+#endif
 	binary_name = argv[0];
 	homedir_path = getenv("HOME");
 
-	signal(SIGSEGV, handle_sigsegv);
 	signal(SIGCHLD, handle_sigchld);
 	signal(SIGUSR1, handle_sigusr1);
 
@@ -116,6 +119,12 @@ int main(int argc, char **argv) {
 	screen_manage_existing();
 	options_clean();
 
+	/*
+	 * don't reg this 'till now, so we won't get in an infinite loop
+	 * during startup.
+	 */
+	signal(SIGSEGV, handle_sigsegv);
+
 	/*
 	 * call the main event loop; the loop runs processing on incoming
 	 * X events until something sets restart_flag.  whatever set
