@@ -262,10 +262,15 @@ void plugin_start() {
 
 /* call the xevent handler for a plugin */
 void plugin_handle_event(plugin_t *plugin, XEvent *e) {
+	plugin_t *oldthis;
+
 	if (!plugin->xevent_handler)
 		return;
+	oldthis = plugin_this;
+	plugin_this = plugin;
 	if (plugin->xevent_handler(e) == PLUGIN_UNLOAD)
 		plugin_unload(plugin);
+	plugin_this = oldthis;
 }
 
 /* add a callback handler for a plugin */
@@ -341,7 +346,8 @@ static int callhandler(callback_t *callback, int argc, va_list ap) {
 		break;
 	default:
 		trace("%d arguments is currently too many...", argc);
-		assert((ret = 0));
+		assert(0);
+		ret = PLUGIN_OK;
 	}
 
 	/*
