@@ -137,10 +137,14 @@ static screen_t *screen_manage(int num) {
 	gcvalues.function = GXcopy;
 	gcvalues.foreground = clr.pixel;
 	gcvalues.background = WhitePixel(display, screen->num);
+#ifdef I18N
+	screen->titlegc = XCreateGC(display, screen->root, GCForeground | GCBackground
+		| GCFunction, &gcvalues);
+#else        
 	gcvalues.font = titlefont->fid;
 	screen->titlegc = XCreateGC(display, screen->root, GCForeground | GCBackground
 		| GCFont | GCFunction, &gcvalues);
-
+#endif
 	return screen;
 }
 
