@@ -108,6 +108,11 @@
 #include "xinerama.h"
 #include "options.h"
 
-#define VERSION		"0.0.4"
+/* internationalization */
+#ifdef I18N
+# include "i18n.h"
+#endif
+
+#define VERSION		"0.0.5"
 
 #endif
